/*
 * Copyright (C) 2008 Emweb bvba, Kessel-Lo, Belgium.
 *
 * See the LICENSE file for terms of use.
 */

#include <math.h>
#include <fstream>

#include "ChartsExample.h"
#include "CsvUtil.h"

#include <Wt/Chart/WAxisSliderWidget>
#include <Wt/WApplication>
#include <Wt/WDate>
#include <Wt/WEnvironment>
#include <Wt/WItemDelegate>
#include <Wt/WStandardItemModel>
#include <Wt/WText>

#include <Wt/WBorderLayout>
#include <Wt/WFitLayout>

#include <Wt/WStandardItem>
#include <Wt/WTableView>
#include <Wt/WPushButton>
#include <Wt/Chart/WCartesianChart>
#include <Wt/Chart/WPieChart>
#include <Wt/WTimer>
#include <Wt/WDialog>

using namespace Wt;
using namespace Wt::Chart;


namespace {
    

    
  /*
   * A standard item which converts text edits to numbers
   */
  class NumericItem : public WStandardItem {
  public:
    virtual NumericItem *clone() const {
      return new NumericItem();
    }

    virtual void setData(const boost::any &data, int role = UserRole) {
      boost::any dt;

      if (role == EditRole) {
	std::string s = Wt::asString(data).toUTF8();
	char *endptr;
	double d = strtod(s.c_str(), &endptr);
	if (*endptr == 0)
	  dt = boost::any(d);
	else
	  dt = data;
      }

      WStandardItem::setData(data, role);
    }
  };

  /*
   * Reads a CSV file as an (editable) standard item model.
   */
  WAbstractItemModel *readCsvFile(const std::string &fname,
				  WContainerWidget *parent)
  {
    WStandardItemModel *model = new WStandardItemModel(0, 0, parent);
    model->setItemPrototype(new NumericItem());
    std::ifstream f(fname.c_str());

    if (f) {
      readFromCsv(f, model);

      for (int row = 0; row < model->rowCount(); ++row)
	for (int col = 0; col < model->columnCount(); ++col) {
	  model->item(row, col)->setFlags(ItemIsSelectable | ItemIsEditable);

	}

      return model;
    } else {
      WString error(WString::tr("error-missing-data"));
      error.arg(fname, UTF8);
      new WText(error, parent);
      return 0;
    }
  }
}


ChartsExample::ChartsExample(WContainerWidget *root)
  : WContainerWidget(root)
{
  new WText(WString::tr("introduction"), this);
  //document.getElementById('myframe').documentWindow.location.href

  new ScatterPlotExample(this);
   // Wt::WContainerWidget *container = new Wt::WContainerWidget(this);
/*    
    auto displayText = new Wt::WText("empytxt");
    std::stringstream ss;
    ss <<  "var it = document.getElementById('myframe');console.log(it);console.log(it.src);";
    
    auto text = new Wt::WText("ffffff Hello <i>dear</i> visitor. <iframe  id=\"myframe\"   onLoad=\"alert(window.location.href);"+ss.str()+"\" src=\"https://www.bruker.com\"> "
  "<p>Your browser does not support iframes.</p>"
"</iframe>", Wt::XHTMLUnsafeText);

  text->setTextFormat(Wt::XHTMLUnsafeText);
  container->addWidget(text);
  container->addWidget(displayText);*/
}

void  ScatterPlotExample::layoutSizeChanged (int width, int height) 
{
    new WText(WString::tr("layoutSizeChanged!!"), this);
}

ScatterPlotExample::ScatterPlotExample(WContainerWidget *parent):
  WContainerWidget(parent)
{

  this->setLayoutSizeAware(true);

  WStandardItemModel *model = new WStandardItemModel(70, 4, this);
  model->setItemPrototype(new NumericItem());
  model->setHeaderData(0, WString("A"));
  model->setHeaderData(1, WString("B")); 
  model->setHeaderData(2, WString("C"));

  for (unsigned i = 0; i < 70; ++i) {
    double x = (static_cast<double>(i) - 20) / 4;

    model->setData(i, 0, x);
    
    double shiftup = 10;
    
    if(i <10)
    {
      model->setData(i, 1, -17 + shiftup);
    }
    else if(i>60)
    {
      model->setData(i, 1,  -17 + shiftup);
    }
    else
    {
      model->setData(i, 1,  -17 + sin(x*10)-0.2 + shiftup);
    }
    
    model->setData(i, 3, static_cast<double>(i)/4.0);
    
    model->setData(i, 2, cos(x)*20 + shiftup);
  }
 
  /*
   * Create the scatter plot.
   */
  WCartesianChart *chart = new WCartesianChart(this);
  
  chart->setModel(model);        // set the model
  chart->setXSeriesColumn(0);    // set the column that holds the X data
  //chart->setLegendEneabled(true); // enable the legend

  chart->setType(ScatterPlot);   // set type to ScatterPlot

  // Typically, for mathematical functions, you want the axes to cross
  // at the 0 mark:
  chart->axis(XAxis).setLocation(BothSides);
  chart->axis(YAxis).setLocation(BothSides);

  // Automatically layout chart (space for axes, legend, ...)
  chart->setAutoLayoutEnabled();

  // Add the curves
  WDataSeries s(1, CurveSeries);
  s.setPen(Wt::WPen(Wt::green));
  chart->addSeries(s);
  
  WDataSeries s2(2, CurveSeries);
  chart->addSeries(s2);
  
    
  WDataSeries s3(3, CurveSeries);
  chart->addSeries(s3);

  chart->resize(800, 300); // WPaintedWidget must be given explicit size

  chart->setMargin(10, Top | Bottom);            // add margin vertically
  chart->setMargin(WLength::Auto, Left | Right); // center horizontally
  chart->setZoomEnabled(true);
  chart->setPanEnabled(true);
  
  
  auto curve = Wt::Chart::CurveLabel(*chart->series()[0],WPointF(1.5,sin(1.5)), "test2");
  chart->setCurveManipulationEnabled(true);
  chart->setSeriesSelectionEnabled(true);
  
  chart->addCurveLabel(std::move(curve));
  
    // Add a WAxisSliderWidget for the chart using the data series for column 2

  //Wt::Chart::WAxisSliderWidget *sliderWidget = new Wt::Chart::WAxisSliderWidget(chart->series()[0], parent);

  Wt::Chart::WAxisSliderWidget *sliderWidget = new Wt::Chart::WAxisSliderWidget( parent);
  sliderWidget->resize(800, 80);
  sliderWidget->setSelectionAreaPadding(40, Wt::Left | Wt::Right);
  sliderWidget->setMargin(Wt::WLength::Auto, Wt::Left | Wt::Right); // Center horizontally
  sliderWidget->setSeries(chart->series()[1]);
  sliderWidget->setSeriesPen(chart->series()[1]->pen());
  sliderWidget->setYAxisZoomEnabled(true);

  auto buttonA = new WPushButton("test Dialog with ObjectName",parent);
  auto button0 = new WPushButton("test Dialog",parent);
  auto button = new WPushButton("test zoom in Y",parent);
  auto button2 = new WPushButton("test zoom out Y",parent);
  auto button3 = new WPushButton("test Offset",parent);
  auto button4 = new WPushButton("test scale",parent);
  auto button5 = new WPushButton("test scale 2",parent);
  auto button6 = new WPushButton("test jump axis",parent);
  auto button7 = new WPushButton("test axis changes",parent);
  auto button8 = new WPushButton("test resize",parent);
    
  button4->setHidden(true);
  button5->setHidden(true);
  buttonA->setHidden(true);
  button0->setHidden(true);
  button5->setHidden(true);
  button->setHidden(true);
  button3->setHidden(true);
  button2->setHidden(true);
  button6->setHidden(true);
  button7->setHidden(true);

  auto stly = parent->decorationStyle();
  stly.setBackgroundColor(Wt::gray);
  parent->setDecorationStyle(stly);
  
  buttonA->clicked().connect(std::bind([parent](){
    
    auto dia = new WDialog("Personalia", parent);
    new Wt::WText("Enter your name: ", dia->contents());
    new Wt::WLineEdit(dia->contents());
    new Wt::WBreak(dia->contents());
    new Wt::WPushButton("Ok", dia->contents());
    
    dia->setObjectName("XXX"); //!!!!
    dia->setModal(true);
    dia->setTransient(true);
    dia->show();
  }));
  
    button0->clicked().connect(std::bind([parent](){
    
    auto dia = new WDialog("Personalia", parent);
    new Wt::WText("Enter your name: ", dia->contents());
    new Wt::WLineEdit(dia->contents());
    new Wt::WBreak(dia->contents());
    new Wt::WPushButton("Ok", dia->contents());
    
    dia->setModal(true);
    dia->setTransient(true);
    dia->show();
  }));
  
  
  button->clicked().connect(std::bind([chart](){
    chart->axis(Wt::Chart::YAxis).setRange(-0.2,0.2); 
    chart->axis(Wt::Chart::YAxis).setZoomRange(-0.2,0.2); 
  }));
  
  button2->clicked().connect(std::bind([chart](){
    chart->axis(Wt::Chart::YAxis).setRange(-3.2,3.2); 
  }));
  
  
  button3->clicked().connect(std::bind([chart]()
  {
    chart->series().at(2)->setOffset(-17);
  }));

  button4->clicked().connect(std::bind([chart]()
  {
    chart->series().at(2)->setScale(0.2);
  }));
  button5->clicked().connect(std::bind([chart]()
  {
    chart->series().at(0)->setScale(0.2);
  }));
  
  button6->clicked().connect(std::bind([chart]()
  {
    chart->axis(Wt::Chart::XAxis).setZoomRange(4,8); 
  }));
  
  int axis_index = 0;
  button7->clicked().connect(std::bind([chart,axis_index,sliderWidget]() mutable
  {
   sliderWidget->setSeries(chart->series()[axis_index]);
    axis_index++;
  }));
  
  
  //this->setObjectName("AAAA"); // uncomment this line, to reproduce the problem on layoutSizeChanged
  
  int addedheight =0 ;
  button8->clicked().connect(std::bind([chart,this, addedheight]() mutable
  {
    this->resize(800,400 + addedheight);
    addedheight++;
  }));
  
}
